@@ -8,7 +8,7 @@ use Devel::hdb::TraceFollow;
 use IO::Socket::INET;
 use IO::File;
 
-our $VERSION = 0.12;
+our $VERSION = 0.13;
 
 sub import {
     my $class = shift;
@@ -161,7 +161,7 @@ them, click the blue circle.
 A watched typeglob will show all the used slots within the glob.  Older
 versions of perl will always create an undefined value in the SCALAR slot.
 The value for the IO slot will be the file descriptor number of the
-filehandle, or undef if it is closed.
+filehandle and its position as reported by sysseek(), or undef if it is closed.
 
 =head3 Key bindings
 
