@@ -5,7 +5,7 @@ use warnings;
 
 use base 'Devel::hdb::App::Base';
 
-our $VERSION = '0.23_15';
+use Devel::hdb::Response;
 
 __PACKAGE__->add_route('post', '/announce_child', \&announce_child);
 
@@ -15,19 +15,18 @@ sub announce_child {
     my $req = Plack::Request->new($env);
     my $child_pid = $req->param('pid');
     my $child_uri = $req->param('uri');
-    my $gui_uri = $req->param('gui');
 
-    $app->enqueue_event({
-        type => 'fork',
-        pid => $child_pid,
-        href => $child_uri,
-        gui_href => $gui_uri,
-        continue_href => "${child_uri}/continue?nostop=1",
-    });
+    my $resp = Devel::hdb::Response->queue('child_process', $env);
+    $resp->{data} = {
+            pid => $child_pid,
+            uri => $child_uri,
+            run => $child_uri . 'continue?nostop=1'
+        };
 
-    return [204, [], []];
+    return [200, [], []];
 }
 
+
 1;
 
 =pod
@@ -45,11 +44,11 @@ listening at a particular URL.
 
 =over 4
 
-=item POST /announce_child?pid=<pid>&uri=<uri>
+=item POST /announce_child
 
 This route requires two parameters:
   pid   The process ID of the sending process
-  uri   The URL of the debugger of the sending process
+  url   The URL of the debugger of the sending process
 
 After a child process forks, it should contact the parent process' debugger
 at this route to notify the parent what URL it is listening for commands on.
@@ -59,7 +58,7 @@ The parent should then pass that information along to the user.
 
 =head1 SEE ALSO
 
-L<Devel::hdb>
+Devel::hdb
 
 =head1 AUTHOR
 
