@@ -5,7 +5,7 @@ use warnings;
 
 use base 'Devel::hdb::App::Base';
 use Devel::hdb::Response;
-use Devel::hdb::App::Stack qw(_stack);
+use Devel::hdb::App::Stack qw(_serialize_stack);
 
 __PACKAGE__->add_route('get', '/stack', \&stack);
 __PACKAGE__->add_route('get', '/stepin', \&stepin);
@@ -16,23 +16,21 @@ __PACKAGE__->add_route('get', '/continue', \&continue);
 sub stepin {
     my($class, $app, $env) = @_;
 
-    $DB::single=1;
+    $app->step;
     return $class->_delay_stack_return_to_client($app, $env);
 }
 
 sub stepover {
     my($class, $app, $env) = @_;
 
-    $DB::single=1;
-    $DB::step_over_depth = $DB::stack_depth;
+    $app->stepover;
     return $class->_delay_stack_return_to_client($app, $env);
 }
 
 sub stepout {
     my($class, $app, $env) = @_;
 
-    $DB::single=0;
-    $DB::step_over_depth = $DB::stack_depth - 1;
+    $app->stepout;
     return $class->_delay_stack_return_to_client($app, $env);
 }
 
@@ -42,9 +40,9 @@ sub continue {
     my $req = Plack::Request->new($env);
     my $nostop = $req->param('nostop');
 
-    $DB::single=0;
+    $app->continue;
     if ($nostop) {
-        DB->disable_debugger();
+        $app->disable_debugger();
         my $resp = Devel::hdb::Response->new('continue', $env);
         $resp->data({ nostop => 1 });
         $env->{'psgix.harakiri.commit'} = Plack::Util::TRUE;
@@ -69,12 +67,12 @@ sub _delay_stack_return_to_client {
         my $writer = $responder->([ 200, [ 'Content-Type' => 'application/json' ]]);
         $env->{'psgix.harakiri.commit'} = Plack::Util::TRUE;
 
-        DB->long_call( sub {
+        $app->{at_next_breakpoint} = sub {
             my $resp = Devel::hdb::Response->new('stack', $env);
-            $resp->data( $class->_stack($app) );
+            $resp->data( $class->_serialize_stack($app) );
             $writer->write( $resp->encode );
             $writer->close();
-        });
+        };
     };
 }
 
@@ -134,5 +132,5 @@ Anthony Brummett <brummett@cpan.org>
 
 =head1 COPYRIGHT
 
-Copyright 2013, Anthony Brummett.  This module is free software. It may
+Copyright 2014, Anthony Brummett.  This module is free software. It may
 be used, redistributed and/or modified under the same terms as Perl itself.
