@@ -352,12 +352,21 @@ function Debugger(sel) {
             filename = $elt.closest('.program-code').attr('data-filename'),
             lineno = $elt.closest('.code-line').attr('data-lineno'),
             bp = this.breakpointManager.get(filename, lineno),
-            requestData = { f: filename, l: lineno };
+            requestData = { f: filename, l: lineno },
+            url, type;
 
-        requestData.c = bp.condition ? '' : 1;  // toggle
+        // toggle the breakpoint
+        if (bp.condition) {
+            url = 'delete-breakpoint',
+            type = 'GET';
+        } else {
+            url = 'breakpoint',
+            type = 'POST'
+            requestData.c = '1';
+        }
 
-        $.ajax({url: 'breakpoint',
-                type: 'POST',
+        $.ajax({url: url,
+                type: type,
                 dataType: 'json',
                 data: requestData,
                 success: this.messageHandler.consumer,
@@ -503,18 +512,14 @@ function Debugger(sel) {
         var $elt = $(e.currentTarget),
             filename = $elt.closest('[data-filename]').attr('data-filename'),
             lineno = $elt.closest('[data-lineno]').attr('data-lineno'),
-            type = $elt.hasClass('toggle-breakpoint') ? 'condition' : 'action',
+            url = $elt.hasClass('toggle-breakpoint') ? 'breakpoint' : 'action',
             state = $elt.is(':checked'),
             requestData = { f: filename, l: lineno };
 
-        if (type === 'condition') {
-            // ci means condition inactive, so we need to flip the state value to send
-            requestData.ci = state ? 0 : 1;
-        } else {
-            requestData.ai = state ? 0 : 1;
-        }
+        // ci means condition inactive, so we need to flip the state value to send
+        requestData.ci = state ? 0 : 1;
 
-        $.ajax({url: 'breakpoint',
+        $.ajax({url: url,
                 type: 'POST',
                 dataType: 'json',
                 data: requestData,
@@ -529,10 +534,10 @@ function Debugger(sel) {
             filename = $elt.closest('[data-filename]').attr('data-filename'),
             lineno = $elt.closest('[data-lineno]').attr('data-lineno');
 
-        $.ajax({url: 'breakpoint',
-                type: 'POST',
+        $.ajax({url: 'delete-breakpoint',
+                type: 'GET',
                 dataType: 'json',
-                data: { f: filename, l: lineno, c: '', a: '' },
+                data: { f: filename, l: lineno },
                 success: this.messageHandler.consumer,
                 error: this.ajaxError
             });
@@ -1083,8 +1088,8 @@ function Debugger(sel) {
                 });
         };
         loadFile = function(subInfo) {
-            var filename = subInfo.data.file,
-                line     = subInfo.data.start;
+            var filename = subInfo.data.filename,
+                line     = subInfo.data.line;
 
             that.fileManager.loadFile(filename)
                             .done( function($elt) { renderNewFile($elt, filename, line) });
@@ -1133,109 +1138,146 @@ function Debugger(sel) {
     this.initialize();
 
     function BreakpointManager() {
-        this.breakpoints    = {};  // keyed by filename, then by line number
-        this.messageHandler = dbg.messageHandler;
+        breakpoints    = {};  // keyed by filename, then by line number
+        actions        = {};  // keyed by filename, then by line number
+        messageHandler = dbg.messageHandler;
 
-        this.breakpointUpdated = function(data) {
-            var filename    = data.filename,
-                line        = data.lineno,
-                condition   = data.condition,
-                action      = data.action,
-                conditionInactive   = data.condition_inactive,
-                actionInactive      = data.action_inactive;
+        function combinedBreakpointAction(filename, line) {
+            var combined = {};
 
-            var breakpoint = {};
-            breakpoint[line] = {};
-
-            if (condition) {
-                breakpoint[line].condition = condition;
-                breakpoint[line].conditionEnabled = ! conditionInactive;
-            }
-            if (action) {
-                breakpoint[line].action = action;
-                breakpoint[line].actionEnabled = ! actionInactive;
-            }
-
-            if (condition || action) {
-                // breakpoint in a new file
-                if (! (filename in this.breakpoints)) {
-                    this.breakpoints[filename] = {};
+            if (line === undefined) {
+                var actionableCopier = function(storage, key) {
+                    for (var line in storage ) {
+                        combined[line] = combined[line] ? combined[line] : {};
+                        combined[line][key] = storage[key].code;
+                        combined[line][key + 'Enabled'] = storage[key].enabled;
+                    }
+                };
+                actionableCopier(breakpoints[filename], 'condition');
+                actionableCopier(actions[filename], 'action');
+            } else {
+                combined[line] = {};
+                if ((filename in breakpoints) && (line in breakpoints[filename])) {
+                    combined[line].condition = breakpoints[filename][line].code;
+                    combined[line].conditionEnabled = breakpoints[filename][line].enabled;
+                }
+                if ((filename in actions) && (line in actions[filename])) {
+                    combined[line].action = actions[filename][line].code;
+                    combined[line].actionEnabled = actions[filename][line].enabled;
                 }
-                this.breakpoints[filename][line] = breakpoint[line];
-            } else if (filename in this.breakpoints) {
-                // removing a breakpoint + action
-                delete this.breakpoints[filename][line];
             }
-
-            dbg.updateBreakpointList(filename, breakpoint);
-            $('.program-code[data-filename="'+filename+'"]')
-                .trigger('breakpoints-updated', [ breakpoint ]);
+            return combined;
         };
+            
+        function makeUpdateMessageHandler(storage, code_key) {
+            return function(data, is_delete) {
+                var filename    = data.filename,
+                    line        = data.lineno,
+                    code        = data.code,
+                    inactive    = data.inactive;
+
+                if (is_delete) {
+                    delete storage[filename][line];
+                } else {
+                    if (! (filename in storage)) {
+                        storage[filename] = {};
+                    }
+                    storage[filename][line] = {};
+                    storage[filename][line].code = code;
+                    storage[filename][line].enabled = ! inactive;
+                }
+
+                var combinedBreakpoint = combinedBreakpointAction(filename, line);
+                
+                dbg.updateBreakpointList(filename, combinedBreakpoint);
+                $('.program-code[data-filename="'+filename+'"]')
+                    .trigger('breakpoints-updated', [ combinedBreakpoint ]);
+            }
+        };
+        var breakpointUpdated = makeUpdateMessageHandler(breakpoints, 'condition');
+        var actionUpdated = makeUpdateMessageHandler(actions, 'action');
 
         this.markBreakpoints = function(filename, $elt) {
 
-            if (filename in this.breakpoints) {
-                $elt.trigger('breakpoints-updated', this.breakpoints[filename]);
+            if ((filename in breakpoints) || (filename in actions)) {
+                var combinedBreakpoint = combinedBreakpointAction(filename);
+                $elt.trigger('breakpoints-updated', combinedBreakpoint);
             } else {
-                $.ajax({url: 'breakpoints',
-                        type: 'GET',
-                        data: { f: filename },
-                        dataType: 'json',
-                        success: this.messageHandler.consumer
-                    });
+                ['breakpoints','actions'].forEach(function(route) {
+                    $.ajax({url: route,
+                            type: 'GET',
+                            data: { f: filename },
+                            dataType: 'json',
+                            success: this.messageHandler.consumer
+                        });
+                });
             }
         };
         this.get = function(filename, lineno) {
-           if (filename in this.breakpoints) {
-                return this.breakpoints[filename][lineno] || {};
-            } else {
-                return {};
-            }
+            var bp = combinedBreakpointAction(filename, lineno);
+            return (lineno in bp) ? bp[lineno] : {};
         };
         this.set = function(params) {
             var filename = params.filename,
                 lineno = params.lineno,
-                postData = { f: filename, l: lineno };
+                toSend = [];
 
             // Save the condition now in case other parts of the app need to
             // see the change before the browser's event loop gets a chance to
             // process the ajax send and receive the reply
-            this.breakpoints[filename] = this.breakpoints[filename] || {};
-            this.breakpoints[filename][lineno] = this.breakpoints[filename][lineno] || {};
+            breakpoints[filename] = breakpoints[filename] || {};
+            breakpoints[filename][lineno] = breakpoints[filename][lineno] || {};
+            actions[filename] = actions[filename] || {};
+            actions[filename][lineno] = actions[filename][lineno] || {};
 
-            if ('condition' in params) {
-                this.breakpoints[filename][lineno].condition = params.condition;
-                postData.c = params.condition;
-                if (! ('conditionEnabled' in params)) {
-                    // If enable/disable wasn't given, assume enabled
-                    this.breakpoints[filename][lineno].conditionEnabled = true;
+            if (('condition' in params) || ('conditionEnabled' in params)) {
+                var breakpoint = { f: filename, l: lineno };
+
+                if ('condition' in params) {
+                    breakpoints[filename][lineno].condition = params.condition;
+                    breakpoint.c = params.condition;
+                    if (! ('conditionEnabled' in params)) {
+                        // If enable/disable wasn't given, assume enabled
+                        breakpoints[filename][lineno].conditionEnabled = true;
+                    }
                 }
-            }
-            if ('conditionEnabled' in params) {
-                this.breakpoints[filename][lineno].conditionEnabled = params.conditionEnabled;
-                postData.ci = ! params.conditionEnabled;
-            }
-            if ('action' in params) {
-                this.breakpoints[filename][lineno].action = params.action;
-                postData.a = params.action;
-                if (! ('actionEnabled' in params)) {
-                    // If enable/disable wasn't given, assume enabled
-                    this.breakpoints[filename][lineno].actionEnabled = true;
+                if ('conditionEnabled' in params) {
+                    breakpoints[filename][lineno].conditionEnabled = params.conditionEnabled;
+                    breakpoint.ci = ! params.conditionEnabled;
                 }
+                toSend.push({ route: 'breakpoint', params: breakpoint });
             }
-            if ('actionEnabled' in params) {
-                this.breakpoints[filename][lineno].actionEnabled = params.actionEnabled;
-                postData.ci = ! params.actionEnabled;
+            if (('action' in params) || ('actionEnabled' in params)) {
+                var action = { f: filename, l: lineno };
+
+                if ('action' in params) {
+                    actions[filename][lineno].action = params.action;
+                    action.c = params.action;
+                    if (! ('actionEnabled' in params)) {
+                        // If enable/disable wasn't given, assume enabled
+                        actions[filename][lineno].actionEnabled = true;
+                    }
+                }
+                if ('actionEnabled' in params) {
+                    actions[filename][lineno].actionEnabled = params.actionEnabled;
+                    action.ci = ! params.actionEnabled;
+                }
+                toSend.push({ route: 'action', params: action });
             }
 
-            $.ajax({url: 'breakpoint',
-                    type: 'POST',
-                    data: postData,
-                    success: this.messageHandler.consumer
-                });
+            toSend.forEach(function(msg) {
+                $.ajax({url: msg.route,
+                        type: 'POST',
+                        data: msg.params,
+                        success: this.messageHandler.consumer
+                    });
+            });
         };
 
-        this.messageHandler.addHandler('breakpoint', this.breakpointUpdated.bind(this))
+        messageHandler.addHandler('breakpoint', breakpointUpdated)
+        messageHandler.addHandler('delete-breakpoint', function(data) { breakpointUpdated(data, 1) });
+        messageHandler.addHandler('action', actionUpdated);
+        messageHandler.addHandler('delete-action', function(data) { actionUpdated(data, 1) });
     };
 }
 
